﻿
--[[

格式说明：

{
	下拉选项标题,
	{
		singlePosFormatRule = 单点取色生成格式函数,
		multiPosFormatRule = 多点取色生成格式函数,
		currentPosInfo = 鼠标所指当前点的格式生成函数（面板上那个随着鼠标移动时刻在变化的内容）,
		makeScriptRule = { -- 自定义脚本规则
			生成第 1 个文本框内容的函数,
			生成第 2 个文本框内容的函数,
			生成第 3 个文本框内容的函数,
		},
	},
}


singlePosFormatRule、multiPosFormatRule 参数说明：
	p：当前点信息，包括 x、y、c、r、g、b、num
	a：坐标缓冲位 A 的信息，包括 x、y
	s：坐标缓冲位 S 的信息，包括 x、y
	x：坐标缓冲位 X 的信息，包括 x、y
	c：坐标缓冲位 C 的信息，包括 x、y


makeScriptRule 参数说明：
	poslist = {
		p,
		p,
		p,
		...
		a = {x, y},
		s = {x, y},
		x = {x, y},
		c = {x, y},
	}

--]]


function make_FMC(poslist)
	local ret = ""
	local firstPos
	for i,currentPos in ipairs(poslist) do
		if i==1 then
			firstPos = currentPos
			ret = ret..string.format("0x%06x, \"", firstPos.c)
		else
			ret = ret..string.format("%d|%d|0x%06x", currentPos.x - firstPos.x, currentPos.y - firstPos.y, currentPos.c)
			if i~=#poslist then
				ret = ret..","
			end
		end
	end
	return ret..'"'
end

function make_findMultiColorInRegionFuzzy(poslist)
	return string.format("x,y = findMultiColorInRegionFuzzy( %s, 90, %d, %d, %d, %d)",
															make_FMC(poslist),
															poslist.a.x,
															poslist.a.y,
															poslist.s.x,
															poslist.s.y)
end

function make_currentPosInfo(fmt)
	return function(p, a, s, x, c)
		return string.format("坐标: (%4d, %4d)\r\n颜色值: 0x%06x\r\nRGB值: (%3d, %3d, %3d)\r\n"..fmt, p.x, p.y, p.c, p.r, p.g, p.b, p.x, p.y, p.c)
	end
end

function z_cmpColor(clr1,clr2)
	local abs = math.abs
    local rd = (0xFF-abs((clr1/0x10000)-(clr2/0x10000)))/0xFF
    local gd = (0xFF-abs(((clr1%0x10000)/0x100)-((clr2%0x10000)/0x100)))/0xFF
    local bd = (0xFF-abs((clr1%0x100)-(clr2%0x100)))/0xFF
    return math.ceil(rd*gd*bd*100)
end

function isColor(x,y,c,s)
	local fl,abs = math.floor,math.abs
	s = fl(0xff*(100-s)*0.01)
	local r,g,b = fl(c/0x10000),fl(c%0x10000/0x100),fl(c%0x100)
	local rr,gg,bb = getColorRGB(x,y)
	if abs(r-rr)<s and abs(g-gg)<s and abs(b-bb)<s then
		return true
	end
end

function capMatrix(tl, br, color, csim, xstep, ystep)
	xstep = xstep or 1
	if xstep<=0 then
		xstep = 1
	end
	ystep = ystep or 1
	if ystep<=0 then
		ystep = 1
	end
	local ret = ""
	for cy = tl.y, br.y, ystep do
		for cx = tl.x, br.x, xstep do
			if (z_cmpColor(getColor(cx, cy), color)>csim) then
				ret = ret.."1"
			else
				ret = ret.."0"
			end
		end
		ret = ret.."\n"
	end
	return ret
end


--[[
-------------------------------------------------------------------
	老版本兼容格式
--]]

function to_old_commonFormat(fmtString, a, s)
	local tos = tostring
	local fmt = string.format
	local ret = fmtString
	ret = ret:gsub( "#SX#", 	fmt("%4d", a.x))
	ret = ret:gsub( "#SY#", 	fmt("%4d", a.y))
	ret = ret:gsub( "#EX#", 	fmt("%4d", s.x))
	ret = ret:gsub( "#EY#", 	fmt("%4d", s.y))
	ret = ret:gsub("#0SX#", 	fmt( "%d", a.x))
	ret = ret:gsub("#0SY#", 	fmt( "%d", a.y))
	ret = ret:gsub("#0EX#", 	fmt( "%d", s.x))
	ret = ret:gsub("#0EY#", 	fmt( "%d", s.y))
	ret = ret:gsub( "#CR#", 	"\r")
	ret = ret:gsub( "#LF#", 	"\n")
	ret = ret:gsub( "#SP#", 	" ")
	ret = ret:gsub(  "#T#", 	"\t")
	return ret
end

function to_old_posFormat(fmtString, p)
	local tos = tostring
	local fmt = string.format
	local ret = fmtString
	ret = ret:gsub(  "#X#", 	fmt("%4d", p.x))
	ret = ret:gsub(  "#Y#", 	fmt("%4d", p.y))
	ret = ret:gsub(  "#R#", 	fmt("%3d", p.r))
	ret = ret:gsub(  "#G#", 	fmt("%3d", p.g))
	ret = ret:gsub(  "#B#", 	fmt("%3d", p.b))
	ret = ret:gsub( "#0X#", 	fmt( "%d", p.x))
	ret = ret:gsub( "#0Y#", 	fmt( "%d", p.y))
	ret = ret:gsub( "#0R#", 	fmt( "%d", p.r))
	ret = ret:gsub( "#0G#", 	fmt( "%d", p.g))
	ret = ret:gsub( "#0B#", 	fmt( "%d", p.b))
	ret = ret:gsub( "#HR#", 	fmt("%02x", p.r))
	ret = ret:gsub( "#HG#", 	fmt("%02x", p.g))
	ret = ret:gsub( "#HB#", 	fmt("%02x", p.b))
	ret = ret:gsub(  "#C#", 	fmt("0x%06x", p.c))
	return ret
end

function make_old_singlePosFormatRule(fmtString)
	return (function(p, a, s, x, c)
		return to_old_posFormat(
			to_old_commonFormat(fmtString, a, s),
			p)
	end)
end

make_old_multiPosFormatRule	= make_old_singlePosFormatRule
make_old_currentPosInfo		= make_old_singlePosFormatRule

function make_old_makeScriptRule(fmtPre, fmtPos, fmtSep, fmtFix)
	return (function(poslist)
		local fmc = make_FMC(poslist)
		local ret = to_old_commonFormat(fmtPre, poslist.a, poslist.s):gsub("#FMC#", fmc)
		local sep = to_old_commonFormat(fmtSep, poslist.a, poslist.s):gsub("#FMC#", fmc)
		local fix = to_old_commonFormat(fmtFix, poslist.a, poslist.s):gsub("#FMC#", fmc)
		for i,currentPos in ipairs(poslist) do
			ret = ret..to_old_posFormat(to_old_commonFormat(fmtPos, poslist.a, poslist.s), currentPos)
			if i~=#poslist then
				ret = ret..sep
			end
		end
		return ret..fix
	end)
end

--[[
	老版本兼容格式
-------------------------------------------------------------------
--]]


local config = {
	remoteDelay = 600,
	{"X, Y, Color",
		{
			singlePosFormatRule = (function(p, a, s, x, c)
				return string.format(" %4d, %4d, 0x%06x ", p.x, p.y, p.c)
			end),
			multiPosFormatRule = (function(p, a, s, x, c)
				return string.format(" %4d, %4d, 0x%06x \n", p.x, p.y, p.c)
			end),
			currentPosInfo = make_currentPosInfo(" %4d, %4d, 0x%06x "),
			makeScriptRule = {
				(function(poslist)
					local ret = "{\n"
					for _,currentPos in ipairs(poslist) do
						ret = ret..string.format("\t{ %4d, %4d, 0x%06x},\n", currentPos.x, currentPos.y, currentPos.c)
					end
					return ret.."}"
				end),
				(function(poslist)
					local ret = "if ("
					for i,currentPos in ipairs(poslist) do
						ret = ret..string.format("isColor(%4d, %4d, 0x%06x)", currentPos.x, currentPos.y, currentPos.c)
						if i~=#poslist then
							ret = ret.." and "
						end
					end
					return ret..") then"
				end),
				make_findMultiColorInRegionFuzzy,
			},
		},
	},
	{"{X, Y, Color},",
		{
			singlePosFormatRule = (function(p, a, s, x, c)
				return string.format("{ %4d, %4d, 0x%06x }", p.x, p.y, p.c)
			end),
			multiPosFormatRule = (function(p, a, s, x, c)
				return string.format("{ %4d, %4d, 0x%06x },\n", p.x, p.y, p.c)
			end),
			currentPosInfo = make_currentPosInfo("{ %4d, %4d, 0x%06x }"),
			makeScriptRule = {
				(function(poslist)
					local ret = "{\n"
					for _,currentPos in ipairs(poslist) do
						ret = ret..string.format("\t{ %4d, %4d, 0x%06x},\n", currentPos.x, currentPos.y, currentPos.c)
					end
					return ret.."}"
				end),
				(function(poslist)
					local ret = "if ("
					for i,currentPos in ipairs(poslist) do
						ret = ret..string.format("isColor(%4d, %4d, 0x%06x)", currentPos.x, currentPos.y, currentPos.c)
						if i~=#poslist then
							ret = ret.." and "
						end
					end
					return ret..") then"
				end),
				make_findMultiColorInRegionFuzzy,
			},
		},
	},
	{"pos.new(X, Y, Color),",
		{
			singlePosFormatRule = (function(p, a, s, x, c)
				return string.format("pos.new( %4d, %4d, 0x%06x)", p.x, p.y, p.c)
			end),
			multiPosFormatRule = (function(p, a, s, x, c)
				return string.format("pos.new( %4d, %4d, 0x%06x ),\n", p.x, p.y, p.c)
			end),
			currentPosInfo = make_currentPosInfo("pos.new( %4d, %4d, 0x%06x)"),
			makeScriptRule = {
				(function(poslist)
					local ret = "{\n\tcsim = 90,\n"
					ret = ret..string.format("\ttl = pos.new(%d, %d),\n", poslist.a.x, poslist.a.y)
					ret = ret..string.format("\tbr = pos.new(%d, %d),\n", poslist.s.x, poslist.s.y)
					for _,currentPos in ipairs(poslist) do
						ret = ret..string.format("\tpos.new( %4d, %4d, 0x%06x),\n", currentPos.x, currentPos.y, currentPos.c)
					end
					return ret.."}"
				end),
				(function(poslist)
					local ret = "{\n\tcsim = 90,\n\tdelay = 100,\n\tclkdeep = 100,\n\tlabel = \"\",\n"
					for _,currentPos in ipairs(poslist) do
						ret = ret..string.format("\tpos.new( %4d, %4d, 0x%06x),\n", currentPos.x, currentPos.y, currentPos.c)
					end
					return ret.."}"
				end),
				make_findMultiColorInRegionFuzzy,
			},
		},
	},
	{"精简去空格优化版RGB - By 桃桃",
		{
			singlePosFormatRule = (function(p, a, s, x, c)
				return string.format("%d,%d,%d,%d,%d", p.x, p.y, p.r, p.g, p.b)
			end),
			multiPosFormatRule = (function(p, a, s, x, c)
				return string.format("%d,%d,%d,%d,%d\n", p.x, p.y, p.r, p.g, p.b)
			end),
			currentPosInfo = (function(p, a, s, x, c)
				return string.format("X:%d Y:%d \r\nR:%d G:%d B:%d \r\nC:0x%06x", p.x, p.y, p.r, p.g, p.b, p.c)
			end),
			makeScriptRule = {
				(function(poslist)
					local ret = "{\r\n"
					for _,currentPos in ipairs(poslist) do
						ret = ret..string.format("\t{%d,%d,%d,%d,%d},\r\n", currentPos.x, currentPos.y, currentPos.r, currentPos.g,  currentPos.b)
					end
					return ret.."}"
				end),
				(function(poslist)
					local ret = "if ("
					for i,currentPos in ipairs(poslist) do
						ret = ret..string.format("isColor(%d,%d,%d,%d,%d)", currentPos.x, currentPos.y, currentPos.r, currentPos.g,  currentPos.b)
						if i~=#poslist then
							ret = ret.." and "
						end
					end
					return ret..") then"
				end),
				make_findMultiColorInRegionFuzzy,
			},
		},
	},
	{"精简去空格优化版Hex - By 桃桃",
		{
			singlePosFormatRule = (function(p, a, s, x, c)
				return string.format("%d,%d,0x%06x", p.x, p.y, p.c)
			end),
			multiPosFormatRule = (function(p, a, s, x, c)
				return string.format("%d,%d,0x%06x\n", p.x, p.y, p.c)
			end),
			currentPosInfo = (function(p, a, s, x, c)
				return string.format("X:%d Y:%d \r\nR:%d G:%d B:%d \r\nC:0x%06x", p.x, p.y, p.r, p.g, p.b, p.c)
			end),
			makeScriptRule = {
				(function(poslist)
					local ret = "{\r\n"
					for _,currentPos in ipairs(poslist) do
						ret = ret..string.format("\t{%d,%d,0x%06x},\r\n", currentPos.x, currentPos.y, currentPos.c)
					end
					return ret.."}"
				end),
				(function(poslist)
					local ret = "if ("
					for i,currentPos in ipairs(poslist) do
						ret = ret..string.format("isColor(%d,%d,0x%06x)", currentPos.x, currentPos.y, currentPos.c)
						if i~=#poslist then
							ret = ret.." and "
						end
					end
					return ret..") then"
				end),
				make_findMultiColorInRegionFuzzy,
			},
		},
	},
	{"简易点阵示例(快捷键 ~ )",
	-- 用 Shift + 鼠标左键 框选区域
	-- 使用 ~ 取这个区域中的所有的颜色，与 鼠标所指位置 的颜色进行对比
	-- 相似度大于 90 则设置为 1，否则设置为 0
	-- 输出到 输出窗口 和 剪贴板
		{
			singlePosFormatRule = (function(p, a, s, x, c)
				local matrix = capMatrix(a, s, p.c, 90)
				print(matrix)
				return matrix
			end),
			multiPosFormatRule = (function(p, a, s, x, c)
				return string.format(" %4d, %4d, 0x%06x \n", p.x, p.y, p.c)
			end),
			currentPosInfo = make_currentPosInfo(" %4d, %4d, 0x%06x "),
			makeScriptRule		= {
				make_old_makeScriptRule(
					"{#LF#",
					"#T#{ #X#, #Y#, #C#},#LF#",
					"",
					"}"
				),
				(function(poslist)
					-- 取所有颜色与第一点色做对比
					-- 相似度 为 X 缓冲区 的第 1 个值
					-- 横坐标步 长为 C 缓冲区的 第 1 个值
					-- 纵坐标步 长为 C 缓冲区的 第 2 个值
					if poslist.x.x==0 then
						poslist.x.x = 90
					end
					local matrix = capMatrix(poslist.a, poslist.s, poslist[1].c, poslist.x.x, poslist.c.x, poslist.c.y)
					return matrix
				end),
				make_findMultiColorInRegionFuzzy,
			},
		},
	},
	{"老版本兼容格式示例",
	-- 这只是个例子，如果你喜欢老版本的格式规则，可以通过这个例子重新定义
	-- 老版本格式规则参考说明请往下翻
		{
			singlePosFormatRule	= make_old_singlePosFormatRule("{ #X#, #Y#, #C#}"),
			multiPosFormatRule	= make_old_multiPosFormatRule("{ #X#, #Y#, #C#},#LF#"),
			currentPosInfo		= make_old_currentPosInfo("{ #X#, #Y#, #C#}"),
			makeScriptRule		= {
				make_old_makeScriptRule(
					"{#LF#",
					"#T#{ #X#, #Y#, #C#},#LF#",
					"",
					"}"
				),
				make_old_makeScriptRule(
					"if (",
					"isColor( #X#, #Y#, #C#, 85)",
					"#SP#and#SP#",
					") then"
				),
				make_findMultiColorInRegionFuzzy,
			},
		},
	},
}

return config

--[[
-----------------------------------------------------
老版本的格式规则


自定义取色格式转义标签（全部标签都为 大！写！字！母! ）

	#X#
		代表 X 坐标

	#Y#
		代表 Y 坐标

	#C#
		代表颜色值（6位16进制）

	#R#
		代表颜色中的红色值

	#G#
		代表颜色中的绿色值

	#B#
		代表颜色中的蓝色值

	#HR#
		代表颜色中的红色值（2位16进制）

	#HG#
		代表颜色中的绿色值（2位16进制）

	#HB#
		代表颜色中的蓝色值（2位16进制）


	注意：以上的转义标签只能用于内容部分，不能用于前缀、中缀和后缀之中


	#SX#
		代表设置的起始点（左上）的 X 坐标

	#SY#
		代表设置的起始点（左上）的 Y 坐标

	#EX#
		代表设置的终点（右下）的 X 坐标

	#EY#
		代表设置的终点（右下）的 Y 坐标

	#CR#
		代表回车（\r）

	#LF#
		代表换行（\n）

	#T#
		代表制表符（\t）

	#SP#
		代表空格（在首尾空格被自动剔除的情况下可以使用）

	#0*#
		这不是一个标签，这个扩展用法是关闭部分标签的空格补齐(在部分标签第一个字母前插入一个数字零)
		支持扩展以下标签：
			#X#、#Y#、#R#、#G#、#B#、#SX#、#SY#、#EX#、#EY#
		例如：
			#0X#、#0Y#、#0R#、#0G#、#0B#、#0SX#、#0SY#、#0EX#、#0EY#

	#FMC#
		代表输出 findMultiColorInRegionFuzzy 的前两个参数（此标签仅能用于前缀、中缀、后缀中）
		输出示例：
			0x39383a, "1|-7|0xf6f6f6,3|7|0xf9fafa,0|22|0xf9fafa,-11|2|0xfbfbfc,-36|-29|0xf48315,-18|-31|0xffd528,21|-32|0x4faaf3,37|-32|0x0e6ab8,26|14|0x414042"

	一些示例（如果不知道什么意思请千万别乱复制使用）：
		#X#, #Y#, #C#
		{ #X#, #Y#, #C#},
		pos.new( #X#, #Y#, #C#),
		#X#, #Y#, #R#, #G#, #B#
		{#X#, #Y#, #R#, #G#, #B#},
		(#X#, #Y#, #R#, #G#, #B#)
		(#Y#, #X#, #B#, #G#, #R#)
		(#Y#, #X#, 0x#HB##HG##HR#)

	一些说明：
		坐标值为空格补齐4位
		颜色值为0x开头及0补齐6位
		RGB值为空格补齐3位
		HRGB值为0补齐2位
		任何字段如果首尾需要空格请使用#SP#

-----------------------------------------------------
--]]